Package: BayesFactor
Type: Package
Title: Computation of Bayes Factors for Common Designs
Version: 0.9.12-4.8
Date: 2026-03-08
Authors@R: c(person("Richard D.", "Morey", role = c("aut", "cre", "cph"), email = "richarddmorey@gmail.com"), 
    person("Jeffrey N.", "Rouder", role = "aut", email = "jrouder@uci.edu"), 
    person("Tahira", "Jamil", role = c("ctb","cph"), email = "tahjamil@gmail.com"), 
    person("Simon", "Urbanek", role = c("ctb", "cph"), email = "simon.urbanek@r-project.org"), 
    person("Karl", "Forner", role = c("ctb", "cph"), email = "karl.forner@gmail.com"), 
    person("Alexander", "Ly", role = c("ctb", "cph"), email = "Alexander.Ly.NL@gmail.com"))
Description: A suite of functions for computing
    various Bayes factors for simple designs, including contingency tables,
    one- and two-sample designs, one-way designs, general ANOVA designs, and
    linear regression.
License: GPL-2
VignetteBuilder: knitr
Depends: R (>= 3.2.0), coda, Matrix (>= 1.1-1)
Imports: pbapply, mvtnorm, stringr, utils, graphics, MatrixModels, Rcpp
        (>= 1.1.1), methods, hypergeo
Suggests: doMC, foreach, testthat, knitr, markdown, rmarkdown, arm,
        lme4, xtable, languageR
URL: https://richarddmorey.github.io/BayesFactor/
BugReports: https://github.com/richarddmorey/BayesFactor/issues
LazyLoad: yes
LinkingTo: Rcpp (>= 1.1.1), RcppEigen (>= 0.3.2.2.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-03-07 09:53:12 UTC; saprm3
Author: Richard D. Morey [aut, cre, cph],
  Jeffrey N. Rouder [aut],
  Tahira Jamil [ctb, cph],
  Simon Urbanek [ctb, cph],
  Karl Forner [ctb, cph],
  Alexander Ly [ctb, cph]
Maintainer: Richard D. Morey <richarddmorey@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-07 14:50:02 UTC
