<!--
%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Vignette menu}
\usepackage[utf8]{inputenc}
-->

![alt text](extra/logo.png)

------


BayesFactor manual files
------

```{r echo=FALSE,message=FALSE,results='hide'}
options(markdown.HTML.stylesheet = 'extra/manual.css')
library(knitr)
```

* [Main manual](manual.html)
* [Posterior odds and probabilities](odds_probs.html)
* [Prior checks](priors.html)
* [Comparison to arm/lmer](compare_lme4.html)

